/*
 *	File:	BrotWindow.cp
 *	---------------------
 *
 *	BrotWindow is the file which creates the offscreen GWorld, the window itself,
 *	and handles the basic controls for the window.
 *
 *
 */

#include "BrotWindow.h"
#include "ThreadBrot.h"

BrotWindow::BrotWindow()
{
	CreateWindow(kNormalWindow);
}

BrotWindow::~BrotWindow()
{
}

	
WindowPtr
BrotWindow::MakeNewWindow(WindowPtr behindWindow)
{
	Parameterics		p;
	
	me = GetNewCWindow(128,nil,behindWindow);
	if( me)
	{
		SetWTitle( me, "\pThreaded Mandelbrot");
		ShowWindow( me);
		
		osImageArea = me->portRect;

		if( MakeGWorld( me))
		{
			gTopLeftC.real = -2.0;
			gTopLeftC.imag = 1.25;
			gTopLeftP.h = 0;
			gTopLeftP.v = 0;
			gScale = 2.5 / (_Float) (osImageArea.right - osImageArea.left);
			
			MakeWholeWindowThread( &gTopLeftC, gScale);
			
			return me;
		}
		else
			Debugger();
	}
	return me;
}


OSErr	BrotWindow::MakeWholeWindowThread( Complex *fp, _Float pscale)
{
	Parameterics		p;
	
	queue = new QueueThreads();
	
	p.botRightP.h		= me->portRect.right;
	p.botRightP.v		= me->portRect.bottom;			
	p.osImage 			= osImage;
	p.osImageArea	 	= osImageArea;
	p.window 			= me;
	p.queue				= queue;
	p.topLeftC 			= *fp;
	p.topLeftP.h		= me->portRect.left;
	p.topLeftP.v		= me->portRect.top;
	p.scale 			= pscale;
	p.bitDepth 			= kOSImageBitDepth;
	p.iterations 		= 250;
	p.frame 			= true;

	(void) new ThreadBrot( &p);
	return noErr;
}

void
BrotWindow::Activate(Boolean /* activating */)
{
//	DrawGrowIcon(fWindow);
}


void
BrotWindow::AdjustCursor(EventRecord * /* anEvent */)
{
}


void
BrotWindow::Draw(void)
{
//	EraseRect(&fWindow->portRect);

//	DrawGrowIcon(fWindow);
}

	
void
BrotWindow::KeyDown(EventRecord * /* anEvent */ )
{
	this->Select();
}


void
BrotWindow::AdjustForNewWindowSize(Rect * /* oldSize */, Rect * /* newSize */)
{
//	DrawGrowIcon(fWindow);	
}


Boolean
BrotWindow::Close(void)
{
	StandardCloseResult	result;
	Str255				title;
	
	Str255			numStr;
	
	NumToString( ggLargestStack, numStr);
	SetWTitle( fWindow, numStr);

	while( !Button())
		;

	GetWTitle(this->fWindow,title);
	result = StandardCloseDocument(LMGetCurApName(),title,false,false);

	if (result != kCancelSaveDocument)
	{
		return TWindow::Close();
	}
	return false;
}


OSErr
BrotWindow::HandleDrag(DragTrackingMessage dragMessage,DragReference theDrag)
{
	RgnHandle	hiliteRgn = NewRgn();
	
	switch (dragMessage)
	{
		case	dragTrackingEnterWindow:
			SetRectRgn( hiliteRgn,
						fWindow->portRect.left,
						fWindow->portRect.top,
						fWindow->portRect.right,
						fWindow->portRect.bottom);
			ShowDragHilite(theDrag,hiliteRgn,true);
			break;
			
		case	dragTrackingLeaveWindow:
			HideDragHilite(theDrag);
			break;
			
		default:
			break;
	}
	
	DisposeRgn(hiliteRgn);
	return(noErr);
}

	
OSErr
BrotWindow::HandleDrop(DragReference /* theDrag */)
{
	return(noErr);
}
